********************************************************
* Minimization of global cropland use based on keeping
* the total global production volume
* 2018-06-25, Nikolay Khabarov
********************************************************

$offlisting $offsymxref $offsymlist
option limrow = 0, limcol = 0, solprint = off, sysout = off;

$setglobal GDXDIR 'H:\CAmod\output'

********************************************************
* load yields and current cropland areas data
********************************************************
set SimuID(*), IrrigID(*), CropID(*), YldAre(*);
$gdxin "%GDXDIR%/02b-yield-area.gdx"
$load  SimuID=dim1
$load IrrigID=dim2
$load  CropID=dim3
$load  YldAre=dim4
parameter D(SimuID,IrrigID,CropID,YldAre), A(SimuID,IrrigID);
$load D
$gdxin


* calculate available cropland per simulation unit and irrigation type
A(SimuID,IrrigID) = sum(CropID,D(SimuID,IrrigID,CropID,'Area'));

* calculate global area per crop (Not-optimized)
parameter GlobCropAreaNop(CropID);
GlobCropAreaNop(CropID) = sum((SimuID,IrrigID), D(SimuID,IrrigID,CropID,'Area'));

********************************************************
* load water content per crop
********************************************************

$gdxin "%GDXDIR%/02a-wc.gdx"
$loadm CropID=dim1

parameter N(CropID);
$load N
$gdxin

* correct yield for water content (dry -> fresh matter) and apply yield correction coefficient
D(SimuID,IrrigID,CropID,'Yield') = D(SimuID,IrrigID,CropID,'Yield') / (1 - N(CropID));

* calculate global production per crop
parameter GlobCropProd(CropID);
GlobCropProd(CropID) = sum((SimuID,IrrigID),
         D(SimuID,IrrigID,CropID,'Area') * D(SimuID,IrrigID,CropID,'Yield')
);

********************************************************
* load global production per crop data (FAO)
********************************************************
$gdxin "%GDXDIR%/02c-fao-production.gdx"
parameter F(CropID);
$load F
$gdxin

********************************************************
* optimization problem: equations
********************************************************

* take only those simulation units under an irrigation type where there is any cropland in data D
set ActiveSimuID(SimuID,IrrigID);
ActiveSimuID(SimuID,IrrigID) = yes $ (A(SimuID,IrrigID) > 0);

variable s(SimuID,IrrigID,CropID), z;
positive variable s;
equations
         total_land_used    'objective to be minimized'
         area_share_limit   'total share per simu and irrigation type must not exceed one'
         inactive_simus     'as the problem is indifferent to it - set the share of used cropland to zero if there is zero cropland there'
         fao_product_crop   'total global production per crop (FAO) that must be met'
;

* providing a starting point
s.l(SimuID,IrrigID,CropID) $ (ActiveSimuID(SimuID,IrrigID)) = D(SimuID,IrrigID,CropID,'Area') / A(SimuID,IrrigID);
s.l(SimuID,IrrigID,CropID) $ (not ActiveSimuID(SimuID,IrrigID)) = 0;

* set parameter for allowed cropland share in each simulation unit
parameter max_allowed_cropland_share;
max_allowed_cropland_share = 1.0;
* 0.667
* 0.80
* 0.85

total_land_used ..
         z =E= sum((SimuID,IrrigID,CropID) $ ActiveSimuID(SimuID,IrrigID),
                 A(SimuID,IrrigID) * s(SimuID,IrrigID,CropID)
         );

area_share_limit(SimuID,IrrigID) ..
         sum(CropID,s(SimuID,IrrigID,CropID)) =L= max_allowed_cropland_share;

inactive_simus(SimuID,IrrigID,CropID) ..
         s(SimuID,IrrigID,CropID) $ (not ActiveSimuID(SimuID,IrrigID)) =E= 0;

fao_product_crop(CropID) ..
         F(CropID) =L= sum((SimuID,IrrigID) $ ActiveSimuID(SimuID,IrrigID),
                 A(SimuID,IrrigID) * s(SimuID,IrrigID,CropID)
               * D(SimuID,IrrigID,CropID,'Yield')
         );

********************************************************
* optimization problem: define
********************************************************
* total FAO production per crop = const
model cropland2min / total_land_used, area_share_limit, inactive_simus, fao_product_crop /;


********************************************************
* optimization problem: solve it
********************************************************
* solution time limit in seconds (1000 default = 0.27 h, 100 000 = 27 hours)
Option ResLim = 100000;
Option SysOut = On;

solve cropland2min using lp minimizing z;

********************************************************
* print the main result - saved cropland by irrig type
********************************************************
parameter area_orig(IrrigID), area_new(IrrigID);
area_orig(IrrigID) = sum(SimuID, A(SimuID,IrrigID));
area_new(IrrigID) = sum((SimuID,CropID), A(SimuID,IrrigID) * s.l(SimuID,IrrigID,CropID));

* Rainfed & Irrigated areas
set IrrAll / Rainfed, Irrigated, IrrigAndRainfed /;
set IrrAllMap(IrrAll, IrrigID) / Rainfed.1, Irrigated.2, IrrigAndRainfed.(1,2) /;

parameter area_change_percent(IrrAll);
area_change_percent(IrrAll) = sum(IrrigID $ IrrAllMap(IrrAll, IrrigID),
                  (area_new(IrrigID) - area_orig(IrrigID)))
         / sum(IrrigID $ IrrAllMap(IrrAll, IrrigID), area_orig(IrrigID))
         * 100.0;

display area_change_percent;

********************************************************
* save full set of useful variables
********************************************************

* getting rid of s.marginal
parameter s_opt(SimuID,IrrigID,CropID);
s_opt(SimuID,IrrigID,CropID) = s.l(SimuID,IrrigID,CropID);

* calculate global area per crop (Optimized)
parameter GlobCropAreaOpt(CropID);
GlobCropAreaOpt(CropID) = sum((SimuID,IrrigID), s_opt(SimuID,IrrigID,CropID) * A(SimuID,IrrigID));

execute_unload "%GDXDIR%/03-optimized.gdx" SimuID IrrigID CropID YldAre IrrAll IrrAllMap ActiveSimuID F D N A area_change_percent s_opt GlobCropAreaNop GlobCropAreaOpt

