## Convert and aggregate crop fraction per simulation unit (simu) to total crop area per 5 arcmin pixel

rm(list=ls(all=TRUE))

setwd("H:/CAmod")

### Load libraries and/or install if necessary
pkg_list <- c(
"readr",
"plyr",
"reshape2",
"raster",
"rasterVis",
"readxl",
"ncdf4",
"RColorBrewer",
"rgdal",
"maps",
"mapdata",
"scico",
"maptools"
)

miss_pkgs <- pkg_list[!(pkg_list %in% installed.packages()[,"Package"])]
if(length(miss_pkgs)) install.packages(miss_pkgs)
lapply(pkg_list, require, character.only = TRUE)

### Load EPIC-IIASA 5 arcmin spatial reference for simulation units
simu_ras_file <- "./auxiliary/simuid5mn.tif"
simu_ras <- raster(simu_ras_file)
simu_ras_df <- as.data.frame(simu_ras, xy=T)

### Load SPAM physical area raster for considered crops
spam_pa_bau_ras <- raster("./auxiliary/spam2005_pa_bau.tif")

### Load crop list
crp_lst <- as.character(read.table(paste0("./data/YIELDxAREA_EPICxSPAMpa_Q95/crop-selection.csv"), sep=",", head=F, stringsAsFactors=F))
crp_lst_long <- as.character(read.table(paste0("./data/YIELDxAREA_EPICxSPAMpa_Q95/crop-selection-long.csv"), sep=",", head=F, stringsAsFactors=F))

### Read GAMS input files
for (j in 1:length(crp_lst))

	{
	
	crp_sel <- crp_lst[j]
	yield_area_infile <- paste0("./data/YIELDxAREA_EPICxSPAMpa_Q95/yield-area-", crp_sel, ".csv")
	yield_area_in_df <- read.table(yield_area_infile, head=F, stringsAsFactors=F, sep=",")
	
	if (j==1) { yield_area_out_df <- yield_area_in_df } else { yield_area_out_df <- rbind(yield_area_out_df, yield_area_in_df) }
	
	}

names(yield_area_out_df) <- c("SimUID", "Irr", "crp", "YD", "Area_crp_irr")

tarea_df <- aggregate(yield_area_out_df[, c("SimUID", "Irr", "Area_crp_irr")], by=list(SimUID=yield_area_out_df$SimUID, Irr=yield_area_out_df$Irr), FUN = sum)
names(tarea_df)[(names(tarea_df)=="Area_crp_irr")] <- "Area_tot_irr"
yield_area_out_df <- merge(yield_area_out_df, tarea_df[, c("SimUID", "Irr", "Area_tot_irr")], by.x=c("SimUID", "Irr"), by.y=c("SimUID", "Irr"), all.x=T)
	
### Load results, merge with input data, and calculate remaining area of cropland
yield_area_scens_df <- yield_area_out_df[, c(1:6)]

gams_scen_file <- paste0("./evaluation/03-optimized.csv")
gams_scen_df <- as.data.frame(read.csv(gams_scen_file, head=F, sep=",", stringsAsFactors=F))
names(gams_scen_df) <- c("SimUID", "Irr", "crp", "area_fac")

yield_area_scens_df <- merge(yield_area_scens_df, gams_scen_df, by.x=c("SimUID", "Irr", "crp"), by.y=c("SimUID", "Irr", "crp"), all.x=T)
yield_area_scens_df$area_opt <- yield_area_scens_df$Area_tot_irr*yield_area_scens_df$area_fac

### Aggregate remaining areas per crop (as Supplementary Table S1 for maximum land sparing scenario)
opt_area_crop_agg_df <- aggregate(yield_area_scens_df$area_opt, by=list(Crop=yield_area_scens_df$crp), FUN=sum, na.rm=T)
opt_area_crop_agg_df$Crop <- mapvalues(opt_area_crop_agg_df$Crop, from=crp_lst, to=crp_lst_long)
names(opt_area_crop_agg_df)[(names(opt_area_crop_agg_df)=="x")] <- "Area"
print(opt_area_crop_agg_df)

### aggregate by SimUID
opt_area_simu_agg_df <- aggregate(yield_area_scens_df[, c("Area_crp_irr", "area_opt")], by=list(SimUID=yield_area_scens_df$SimUID), FUN=sum, na.rm=T)

### Calculate scaling factor per SimU
opt_area_simu_agg_df$area_frac <- opt_area_simu_agg_df$area_opt/opt_area_simu_agg_df$Area_crp_irr

### Create scaling raster
opt_area_scale_ras <- simu_ras
values(opt_area_scale_ras) <- NA
values(opt_area_scale_ras) <- opt_area_simu_agg_df$area_frac[match(values(simu_ras), opt_area_simu_agg_df$SimUID)]

### Produce scaled raster based on SPAM BAU harvested area for considered crops and scaling factor per SimU
opt_area_ras <- spam_pa_bau_ras*opt_area_scale_ras

### Plot map of optimized area
levelplot(opt_area_ras)



